@@ -1,6 +1,8 @@
 package cli
 
-import "github.com/spf13/cobra"
+import (
+	"github.com/spf13/cobra"
+)
 
 func NoArgsOrOneValidArg(cmd *cobra.Command, args []string) error {
 	if len(args) == 0 {
