@@ -3,6 +3,7 @@ package prompt
 import (
 	"strings"
 
+	"github.com/jandedobbeleer/oh-my-posh/src/cache"
 	"github.com/jandedobbeleer/oh-my-posh/src/color"
 	"github.com/jandedobbeleer/oh-my-posh/src/config"
 	"github.com/jandedobbeleer/oh-my-posh/src/regex"
@@ -22,9 +23,9 @@ type Engine struct {
 	rprompt               string
 	Overflow              config.Overflow
 	prompt                strings.Builder
-	currentLineLength     int
 	rpromptLength         int
 	Padding               int
+	currentLineLength     int
 	Plain                 bool
 	forceRender           bool
 }
@@ -41,18 +42,22 @@ const (
 	PREVIEW   = "preview"
 )
 
-func (e *Engine) write(text string) {
-	e.prompt.WriteString(text)
+func (e *Engine) write(txt string) {
+	// Grow capacity proactively if needed
+	if e.prompt.Cap() < e.prompt.Len()+len(txt) {
+		e.prompt.Grow(len(txt) * 2) // Grow by double the needed size to reduce future allocations
+	}
+	e.prompt.WriteString(txt)
 }
 
 func (e *Engine) string() string {
-	text := e.prompt.String()
+	txt := e.prompt.String()
 	e.prompt.Reset()
-	return text
+	return txt
 }
 
 func (e *Engine) canWriteRightBlock(length int, rprompt bool) (int, bool) {
-	if rprompt && (len(e.rprompt) == 0) {
+	if rprompt && (e.rprompt == "") {
 		return 0, false
 	}
 
@@ -88,7 +93,7 @@ func (e *Engine) canWriteRightBlock(length int, rprompt bool) (int, bool) {
 
 func (e *Engine) pwd() {
 	// only print when relevant
-	if len(e.Config.PWD) == 0 {
+	if e.Config.PWD == "" {
 		return
 	}
 
@@ -104,12 +109,8 @@ func (e *Engine) pwd() {
 	}
 
 	// Allow template logic to define when to enable the PWD (when supported)
-	tmpl := &template.Text{
-		Template: e.Config.PWD,
-	}
-
-	pwdType, err := tmpl.Render()
-	if err != nil || len(pwdType) == 0 {
+	pwdType, err := template.Render(e.Config.PWD, nil)
+	if err != nil || pwdType == "" {
 		return
 	}
 
@@ -150,17 +151,12 @@ func (e *Engine) isIterm() bool {
 }
 
 func (e *Engine) shouldFill(filler string, padLength int) (string, bool) {
-	if len(filler) == 0 {
+	if filler == "" {
 		return "", false
 	}
 
-	tmpl := &template.Text{
-		Template: filler,
-		Context:  e,
-	}
-
 	var err error
-	if filler, err = tmpl.Render(); err != nil {
+	if filler, err = template.Render(filler, e); err != nil {
 		return "", false
 	}
 
@@ -174,22 +170,20 @@ func (e *Engine) shouldFill(filler string, padLength int) (string, bool) {
 
 	repeat := padLength / lenFiller
 	unfilled := padLength % lenFiller
-	text := strings.Repeat(filler, repeat) + strings.Repeat(" ", unfilled)
-	return text, true
+	txt := strings.Repeat(filler, repeat) + strings.Repeat(" ", unfilled)
+	return txt, true
 }
 
 func (e *Engine) getTitleTemplateText() string {
-	tmpl := &template.Text{
-		Template: e.Config.ConsoleTitleTemplate,
-	}
-	if text, err := tmpl.Render(); err == nil {
-		return text
+	if txt, err := template.Render(e.Config.ConsoleTitleTemplate, nil); err == nil {
+		return txt
 	}
+
 	return ""
 }
 
 func (e *Engine) renderBlock(block *config.Block, cancelNewline bool) bool {
-	text, length := e.writeBlockSegments(block)
+	txt, length := e.writeBlockSegments(block)
 
 	// do not print anything when we don't have any text unless forced
 	if !block.Force && length == 0 {
@@ -209,7 +203,7 @@ func (e *Engine) renderBlock(block *config.Block, cancelNewline bool) bool {
 	case config.Prompt:
 		if block.Alignment == config.Left {
 			e.currentLineLength += length
-			e.write(text)
+			e.write(txt)
 			return true
 		}
 
@@ -227,7 +221,7 @@ func (e *Engine) renderBlock(block *config.Block, cancelNewline bool) bool {
 				e.writeNewline()
 			case config.Hide:
 				// make sure to fill if needed
-				if padText, OK := e.shouldFill(block.Filler, space+length); OK {
+				if padText, OK := e.shouldFill(block.Filler, space+length-e.currentLineLength); OK {
 					e.write(padText)
 				}
 
@@ -244,20 +238,17 @@ func (e *Engine) renderBlock(block *config.Block, cancelNewline bool) bool {
 		// validate if we have a filler and fill if needed
 		if padText, OK := e.shouldFill(block.Filler, space); OK {
 			e.write(padText)
-			e.write(text)
+			e.write(txt)
 			return true
 		}
 
-		var prompt string
-
 		if space > 0 {
-			prompt += strings.Repeat(" ", space)
+			e.write(strings.Repeat(" ", space))
 		}
 
-		prompt += text
-		e.write(prompt)
+		e.write(txt)
 	case config.RPrompt:
-		e.rprompt = text
+		e.rprompt = txt
 		e.rpromptLength = length
 	}
 
@@ -269,7 +260,7 @@ func (e *Engine) applyPowerShellBleedPatch() {
 	// to avoid the background being printed on the next line
 	// when at the end of the buffer.
 	// See https://github.com/JanDeDobbeleer/oh-my-posh/issues/65
-	if e.Env.Shell() != shell.PWSH && e.Env.Shell() != shell.PWSH5 {
+	if e.Env.Shell() != shell.PWSH {
 		return
 	}
 
@@ -329,7 +320,7 @@ func (e *Engine) writeSeparator(final bool) {
 		e.adjustTrailingDiamondColorOverrides()
 	}
 
-	if isPreviousDiamond && isCurrentDiamond && len(e.activeSegment.LeadingDiamond) == 0 {
+	if isPreviousDiamond && isCurrentDiamond && e.activeSegment.LeadingDiamond == "" {
 		terminal.Write(color.Background, color.ParentBackground, e.previousActiveSegment.TrailingDiamond)
 		return
 	}
@@ -345,7 +336,7 @@ func (e *Engine) writeSeparator(final bool) {
 			return false
 		}
 
-		if isPowerline && len(e.activeSegment.LeadingPowerlineSymbol) == 0 {
+		if isPowerline && e.activeSegment.LeadingPowerlineSymbol == "" {
 			return false
 		}
 
@@ -374,7 +365,7 @@ func (e *Engine) writeSeparator(final bool) {
 	}
 
 	symbol := resolvePowerlineSymbol()
-	if len(symbol) == 0 {
+	if symbol == "" {
 		return
 	}
 
@@ -383,7 +374,7 @@ func (e *Engine) writeSeparator(final bool) {
 		bgColor = color.Transparent
 	}
 
-	if e.activeSegment.ResolveStyle() == config.Diamond && len(e.activeSegment.LeadingDiamond) == 0 {
+	if e.activeSegment.ResolveStyle() == config.Diamond && e.activeSegment.LeadingDiamond == "" {
 		bgColor = color.Background
 	}
 
@@ -400,11 +391,11 @@ func (e *Engine) getPowerlineColor() color.Ansi {
 		return color.Transparent
 	}
 
-	if e.previousActiveSegment.ResolveStyle() == config.Diamond && len(e.previousActiveSegment.TrailingDiamond) == 0 {
+	if e.previousActiveSegment.ResolveStyle() == config.Diamond && e.previousActiveSegment.TrailingDiamond == "" {
 		return e.previousActiveSegment.ResolveBackground()
 	}
 
-	if e.activeSegment.ResolveStyle() == config.Diamond && len(e.activeSegment.LeadingDiamond) == 0 {
+	if e.activeSegment.ResolveStyle() == config.Diamond && e.activeSegment.LeadingDiamond == "" {
 		return e.previousActiveSegment.ResolveBackground()
 	}
 
@@ -421,15 +412,20 @@ func (e *Engine) adjustTrailingDiamondColorOverrides() {
 	// this will still break when using parentBackground and parentForeground as keywords
 	// in a trailing diamond, but let's fix that when it happens as it requires either a rewrite
 	// of the logic for diamonds or storing grandparents as well like one happy family.
-	if e.previousActiveSegment == nil || len(e.previousActiveSegment.TrailingDiamond) == 0 {
+	if e.previousActiveSegment == nil || e.previousActiveSegment.TrailingDiamond == "" {
 		return
 	}
 
-	if !strings.Contains(e.previousActiveSegment.TrailingDiamond, string(color.Background)) && !strings.Contains(e.previousActiveSegment.TrailingDiamond, string(color.Foreground)) {
+	trailingDiamond := e.previousActiveSegment.TrailingDiamond
+	// Optimize: check both conditions in a single pass
+	hasBg := strings.Contains(trailingDiamond, string(color.Background))
+	hasFg := strings.Contains(trailingDiamond, string(color.Foreground))
+
+	if !hasBg && !hasFg {
 		return
 	}
 
-	match := regex.FindNamedRegexMatch(terminal.AnchorRegex, e.previousActiveSegment.TrailingDiamond)
+	match := regex.FindNamedRegexMatch(terminal.AnchorRegex, trailingDiamond)
 	if len(match) == 0 {
 		return
 	}
@@ -475,12 +471,12 @@ func (e *Engine) rectifyTerminalWidth(diff int) {
 // given configuration options, and is ready to print any
 // of the prompt components.
 func New(flags *runtime.Flags) *Engine {
-	flags.Config = config.Path(flags.Config)
-	cfg, _ := config.Load(flags.Config, flags.Shell, flags.Migrate)
-
 	env := &runtime.Terminal{}
 	env.Init(flags)
 
+	_, reload := cache.Get[bool](cache.Device, config.RELOAD)
+	cfg := config.Get(flags.ConfigPath, reload)
+
 	template.Init(env, cfg.Var, cfg.Maps)
 
 	flags.HasExtra = cfg.DebugPrompt != nil ||
@@ -505,8 +501,12 @@ func New(flags *runtime.Flags) *Engine {
 		Env:         env,
 		Plain:       flags.Plain,
 		forceRender: flags.Force || len(env.Getenv("POSH_FORCE_RENDER")) > 0,
+		prompt:      strings.Builder{},
 	}
 
+	// Pre-allocate prompt builder capacity to reduce allocations during rendering
+	eng.prompt.Grow(512) // Start with 512 bytes capacity, will grow as needed
+
 	switch env.Shell() {
 	case shell.XONSH:
 		// In Xonsh, the behavior of wrapping at the end of a prompt line is inconsistent across different operating systems.
@@ -522,7 +522,7 @@ func New(flags *runtime.Flags) *Engine {
 			diff = -2
 		}
 		eng.rectifyTerminalWidth(diff)
-	case shell.PWSH, shell.PWSH5:
+	case shell.PWSH:
 		// when in PowerShell, and force patching the bleed bug
 		// we need to reduce the terminal width by 1 so the last
 		// character isn't cut off by the ANSI escape sequences
