@@ -3,13 +3,14 @@ package shell
 import (
 	"fmt"
 	"os"
-	"slices"
 	"strings"
 	"time"
 
+	"github.com/jandedobbeleer/oh-my-posh/src/cache"
 	"github.com/jandedobbeleer/oh-my-posh/src/log"
 	"github.com/jandedobbeleer/oh-my-posh/src/runtime"
 	"github.com/jandedobbeleer/oh-my-posh/src/runtime/path"
+	"github.com/jandedobbeleer/oh-my-posh/src/text"
 )
 
 const (
@@ -42,10 +43,12 @@ func getExecutablePath(env runtime.Environment) (string, error) {
 }
 
 func Init(env runtime.Environment, feats Features) string {
-	shell := env.Flags().Shell
+	switch env.Flags().Shell {
+	case ELVISH, PWSH:
+		if env.Flags().Shell != ELVISH && !env.Flags().Eval {
+			return PrintInit(env, feats, nil)
+		}
 
-	switch shell {
-	case ELVISH:
 		executable, err := getExecutablePath(env)
 		if err != nil {
 			return noExe
@@ -56,24 +59,31 @@ func Init(env runtime.Environment, feats Features) string {
 			additionalParams += " --strict"
 		}
 
-		config := quotePwshOrElvishStr(env.Flags().Config)
+		config := quotePwshOrElvishStr(env.Flags().ConfigPath)
 		executable = quotePwshOrElvishStr(executable)
-		command := "eval ((external %s) init %s --config=%s --print%s | slurp)"
 
-		return fmt.Sprintf(command, executable, shell, config, additionalParams)
-	case ZSH, BASH, FISH, CMD, XONSH, NU, PWSH, PWSH5:
+		var command string
+
+		switch env.Flags().Shell {
+		case PWSH:
+			command = "(@(& %s init %s --config=%s --print --eval%s) -join \"`n\") | Invoke-Expression"
+		case ELVISH:
+			command = "eval ((external %s) init %s --config=%s --print%s | slurp)"
+		}
+
+		return fmt.Sprintf(command, executable, env.Flags().Shell, config, additionalParams)
+	case ZSH, BASH, FISH, CMD, XONSH, NU:
 		return PrintInit(env, feats, nil)
 	default:
-		return fmt.Sprintf(`echo "%s is not supported by Oh My Posh"`, shell)
+		return fmt.Sprintf(`echo "%s is not supported by Oh My Posh"`, env.Flags().Shell)
 	}
 }
 
 func PrintInit(env runtime.Environment, features Features, startTime *time.Time) string {
-	shell := env.Flags().Shell
-	async := slices.Contains(features, Async)
+	async := features&Async != 0
 
 	if scriptPath, OK := hasScript(env); OK {
-		return sourceInit(env, shell, scriptPath, async)
+		return sourceInit(env, scriptPath, async)
 	}
 
 	executable, err := getExecutablePath(env)
@@ -81,65 +91,49 @@ func PrintInit(env runtime.Environment, features Features, startTime *time.Time)
 		return noExe
 	}
 
-	configFile := env.Flags().Config
 	bashBLEsession = len(env.Getenv("BLE_SESSION_ID")) != 0
 
 	var script string
-	sessionID := env.Flags().SessionID
 
-	switch shell {
-	case PWSH, PWSH5:
+	switch env.Flags().Shell {
+	case PWSH:
 		executable = quotePwshOrElvishStr(executable)
-		configFile = quotePwshOrElvishStr(configFile)
-		sessionID = quotePwshOrElvishStr(sessionID)
 		script = pwshInit
 	case ZSH:
 		executable = QuotePosixStr(executable)
-		configFile = QuotePosixStr(configFile)
-		sessionID = QuotePosixStr(sessionID)
 		script = zshInit
 	case BASH:
 		executable = QuotePosixStr(executable)
-		configFile = QuotePosixStr(configFile)
-		sessionID = QuotePosixStr(sessionID)
 		script = bashInit
 	case FISH:
 		executable = quoteFishStr(executable)
-		configFile = quoteFishStr(configFile)
-		sessionID = quoteFishStr(sessionID)
 		script = fishInit
 	case CMD:
 		executable = escapeLuaStr(executable)
-		configFile = escapeLuaStr(configFile)
-		sessionID = escapeLuaStr(sessionID)
 		script = cmdInit
 	case NU:
 		executable = quoteNuStr(executable)
-		configFile = quoteNuStr(configFile)
-		sessionID = quoteNuStr(sessionID)
 		script = nuInit
 	case ELVISH:
 		executable = quotePwshOrElvishStr(executable)
-		configFile = quotePwshOrElvishStr(configFile)
-		sessionID = quotePwshOrElvishStr(sessionID)
 		script = elvishInit
 	case XONSH:
 		executable = quotePythonStr(executable)
-		configFile = quotePythonStr(configFile)
-		sessionID = quotePythonStr(sessionID)
 		script = xonshInit
 	default:
-		return fmt.Sprintf("echo \"No initialization script available for %s\"", shell)
+		return fmt.Sprintf("echo \"No initialization script available for %s\"", env.Flags().Shell)
 	}
 
 	init := strings.NewReplacer(
 		"::OMP::", executable,
-		"::CONFIG::", configFile,
-		"::SHELL::", shell,
-		"::SESSION_ID::", sessionID,
+		"::SESSION_ID::", cache.SessionID(),
 	).Replace(script)
 
-	shellScript := features.Lines(shell).String(init)
+	shellScript := features.Lines(env.Flags().Shell).String(init)
+
+	if env.Flags().Eval {
+		return fmt.Sprintf("%s\n%s", sessionScript(env.Flags().Shell), shellScript)
+	}
 
 	log.Debug(shellScript)
 
@@ -148,20 +142,21 @@ func PrintInit(env runtime.Environment, features Features, startTime *time.Time)
 		return fmt.Sprintf("echo \"Failed to write init script: %s\"", err.Error())
 	}
 
+	sourceCommand := sourceInit(env, scriptPath, async)
+
 	if !env.Flags().Debug {
-		return sourceInit(env, shell, scriptPath, async)
+		return sourceCommand
 	}
 
-	sourceScript := sourceInit(env, shell, scriptPath, async)
-	if len(sourceScript) != 0 {
-		log.Debug("init source script:", script)
+	if len(sourceCommand) != 0 {
+		log.Debug("init source command:", sourceCommand)
 	}
 
 	return printDebug(env, startTime)
 }
 
 func printDebug(env runtime.Environment, startTime *time.Time) string {
-	var builder strings.Builder
+	builder := text.NewBuilder()
 
 	builder.WriteString(fmt.Sprintf("\n%s %s\n", log.Text("Init duration:").Green().Bold().Plain(), time.Since(*startTime)))
 
@@ -171,9 +166,9 @@ func printDebug(env runtime.Environment, startTime *time.Time) string {
 	return builder.String()
 }
 
-func sourceInit(env runtime.Environment, shell, scriptPath string, async bool) string {
+func sourceInit(env runtime.Environment, scriptPath string, async bool) string {
 	// nushell stores to autoload, no need to return anything
-	if shell == NU {
+	if env.Flags().Shell == NU {
 		return ""
 	}
 
@@ -186,38 +181,62 @@ func sourceInit(env runtime.Environment, shell, scriptPath string, async bool) s
 		}
 	}
 
+	script := sessionScript(env.Flags().Shell)
+
 	if async {
-		return sourceInitAsync(shell, scriptPath)
+		return script + sourceInitAsync(env.Flags().Shell, scriptPath)
 	}
 
-	switch shell {
-	case PWSH, PWSH5:
-		return fmt.Sprintf(`& "%s"`, scriptPath)
-	case ZSH, BASH, XONSH:
-		return fmt.Sprintf(`source '%s'`, scriptPath)
+	switch env.Flags().Shell {
+	case PWSH:
+		script += fmt.Sprintf("& %s", quotePwshOrElvishStr(scriptPath))
+	case ZSH, BASH:
+		script += fmt.Sprintf("source %s", QuotePosixStr(scriptPath))
+	case XONSH:
+		script += fmt.Sprintf("source %s", quotePythonStr(scriptPath))
 	case FISH:
-		return fmt.Sprintf(`source "%s"`, scriptPath)
+		script += fmt.Sprintf("source %s", quoteFishStr(scriptPath))
 	case ELVISH:
-		return fmt.Sprintf(`eval (slurp < '%s')`, scriptPath)
+		script += fmt.Sprintf("eval (slurp < %s)", quotePwshOrElvishStr(scriptPath))
 	case CMD:
-		scriptPath = strings.ReplaceAll(scriptPath, `\`, `\\`)
-		return fmt.Sprintf(`load(io.open('%s', "r"):read("*a"))()`, scriptPath)
+		script += fmt.Sprintf(`load(io.open('%s', "r"):read("*a"))()`, escapeLuaStr(scriptPath))
 	default:
-		return fmt.Sprintf("echo \"No source command available for %s\"", shell)
+		return fmt.Sprintf("echo \"No source command available for %s\"", env.Flags().Shell)
 	}
+
+	return script
 }
 
 func sourceInitAsync(shell, scriptPath string) string {
 	switch shell {
-	case PWSH, PWSH5:
-		return fmt.Sprintf(`function prompt() { & "%s" }`, scriptPath)
+	case PWSH:
+		return fmt.Sprintf("function prompt() { & %s }", quotePwshOrElvishStr(scriptPath))
 	case ZSH:
-		return fmt.Sprintf(`precmd() { source '%s' }`, scriptPath)
+		return fmt.Sprintf("precmd() { source %s }", QuotePosixStr(scriptPath))
 	case BASH:
-		return fmt.Sprintf(`PROMPT_COMMAND='source "%s"'`, scriptPath)
+		command := fmt.Sprintf("source %s", QuotePosixStr(scriptPath))
+		return fmt.Sprintf("PROMPT_COMMAND=%s", QuotePosixStr(command))
 	case FISH:
-		return fmt.Sprintf(`function fish_prompt; source "%s"; end`, scriptPath)
+		return fmt.Sprintf("function fish_prompt; source %s; end", quoteFishStr(scriptPath))
 	default:
 		return ""
 	}
 }
+
+func sessionScript(shell string) string {
+	switch shell {
+	case PWSH:
+		return fmt.Sprintf("$env:POSH_SESSION_ID = \"%s\";", cache.SessionID())
+	case ZSH, BASH:
+		return fmt.Sprintf("export POSH_SESSION_ID=\"%s\";", cache.SessionID())
+	case XONSH:
+		return fmt.Sprintf("$POSH_SESSION_ID = \"%s\";", cache.SessionID())
+	case FISH:
+		return fmt.Sprintf("set --export --global POSH_SESSION_ID \"%s\";", cache.SessionID())
+	case ELVISH:
+		return fmt.Sprintf("set-env POSH_SESSION_ID \"%s\";", cache.SessionID())
+	case CMD:
+		return fmt.Sprintf(`os.setenv('POSH_SESSION_ID', '%s');`, cache.SessionID())
+	}
+	return ""
+}
